# -*- coding: utf-8 -*-

"""

    Copyright (C) 2007 Jörg Kubaile <jk@zettwerk.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""

from Globals import ImageFile, DTMLFile, package_home
from OFS.Folder import Folder

import os, logging

log = logging.getLogger('ZEditArea')
product_home = package_home(globals())

## use own setter and getter to read and write the content of the editable text
from setter import *
from getter import *

## import the needed classes to patch
from Products.PythonScripts.PythonScript import PythonScript
from OFS.DTMLDocument import DTMLDocument
from OFS.DTMLMethod import DTMLMethod
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from OFS.Image import File

## and define them in a list to iterate with the corrosponding getter and setter
classesToPatch = [
    ## (class, getter, setter, default_syntax)
    (PythonScript, getEditAreaPythonScript, setEditAreaPythonScript, 'python'),
    (DTMLDocument, getEditAreaDTMLDocument, setEditAreaDTMLDocument, 'html'),
    (DTMLMethod, getEditAreaDTMLMethod, setEditAreaDTMLMethod, 'html'),
    (ZopePageTemplate, getEditAreaZopePageTemplate, setEditAreaZopePageTemplate, 'html'),
    (File, getEditAreaFile, setEditAreaFile, 'js'),
    ]

## make some noise
log.info('Patching for edit_area usage: \n\t%s' %('\n\t'.join([str(c[0]) for c in classesToPatch])))

## this is the global dtml template for all types
editArea = DTMLFile(os.path.join(product_home, 'www', 'editArea'))

## this do the patch thing
def attachEditAreaToClass(class2pDef):
    """ get the class, and set the needed attributes to enable ZEditArea """

    class2p, getter, setter, default_syntax = class2pDef

    ## adding EditForm tab first
    class2p.manage_options = ({'label': 'EditArea', 'action': 'editArea'},) + class2p.manage_options
    class2p.manage = editArea

    ## add the getter and setter
    setattr(class2p, 'getEditArea', getter)
    setattr(class2p, 'setEditArea', setter) 

    ## make the template file knowable to the class
    setattr(class2p, 'editArea', editArea)

    ## set language and syntax attribute
    setattr(class2p, 'getEditAreaLanguage', getEditAreaLanguage)
    setattr(class2p, 'editAreaSyntax', default_syntax)

    ## add additional folders needed from edit_area
    ## cause i don't want to break up the source folder structure
    ## of edit_area - acquisition helps 
    setattr(class2p, 'langs', Folder('langs'))
    setattr(class2p, 'reg_syntax', Folder('reg_syntax'))
    setattr(class2p, 'images', Folder('images'))
    setattr(class2p, 'plugins', Folder('plugins'))
    
    ## the main js script
    setattr(class2p, 'edit_area_full_with_plugins.js', ImageFile(
        os.path.join(product_home, 'edit_area', 'edit_area_full_with_plugins.js'))
            )
    
    ## different language files
    path = os.path.join(product_home, 'edit_area', 'langs')
    availableLanguages = []
    for lang in os.listdir(path):
        availableLanguages.append(lang.split('.')[0])
        setattr(class2p,
                lang,
                ImageFile(os.path.join(product_home, path, lang)),
                )
    setattr(class2p, 'availableEditAreaLanguages', availableLanguages) ## good to know
    
    ## create folder structure for the plugin languages
    plugins = getattr(class2p, 'plugins')
    setattr(plugins, 'syntax_selection', Folder('syntax_selection'))

    syntax_selection = getattr(plugins, 'syntax_selection')
    setattr(syntax_selection, 'langs', Folder('langs'))

    langs = getattr(syntax_selection, 'langs')

    ## and add them
    path = os.path.join(product_home, 'edit_area', 'plugins', 'syntax_selection', 'langs')
    for lang in os.listdir(path):
        setattr(langs,
                lang,
                ImageFile(os.path.join(product_home, path, lang)),
                )

    ## different syntax files
    path = os.path.join(product_home, 'edit_area', 'reg_syntax')
    for syntax in os.listdir(path):
        setattr(class2p,
                syntax,
                ImageFile(os.path.join(product_home, path, syntax)),
                )
        
    ## images
    path = os.path.join(product_home, 'edit_area', 'images')
    for syntax in os.listdir(path):
        setattr(class2p,
                syntax,
                ImageFile(os.path.join(product_home, path, syntax)),
                )

## calling attach for every class to patch
for class2pDef in classesToPatch:
    attachEditAreaToClass(class2pDef)

